// @ts-nocheck
import React, { useEffect, createRef } from 'react';

import { appendGlobalClassName, cleanupSelected, ON_HOVER, SELECTED } from './utils.js';

const useCapture = true;

const shouldIgnoreElement = (ignoreElement, elementRef, target) => {
  if (elementRef && elementRef.current) {
    return elementRef.current.contains(target) === false;
  }

  if (ignoreElement) {
    return ignoreElement.contains(target) === false;
  }

  return false;
};

const Highlight = ({
  ignoreElement,
  isActiveRef,
  selectedElement,
  onSelect,
  onlySelect,
  children,
  tagName,
}) => {
  const elementRef = createRef();
  const activeRef = isActiveRef && isActiveRef.current;

  useEffect(() => {
    const mouseOutCallback = event => {
      const target: HTMLElement = event.target;
      if (!activeRef) return;
      target.classList.remove(ON_HOVER);
    };

    const mouseOverCallback = event => {
      const target: HTMLElement = event.target;
      if (!activeRef) return;
      if (selectedElement && selectedElement === target) return;
      if (
        typeof onlySelect === 'function' &&
        (!target || !onlySelect(target))
      ) {
        return;
      } else if (
        !onlySelect &&
        shouldIgnoreElement(ignoreElement, elementRef, target)
      )
        return;
      target.classList.add(ON_HOVER);
    };

    const mouseClickCallback = event => {
      const target: HTMLElement = event.target;
      if (!activeRef) return;
      if (
        typeof onlySelect === 'function' &&
        (!target || !onlySelect(target))
      ) {
        return;
      } else if (
        !onlySelect &&
        shouldIgnoreElement(ignoreElement, elementRef, target)
      )
        return;

      event.preventDefault();
      event.stopPropagation();

      cleanupSelected();
      target.classList.add(SELECTED);
      if (onSelect) onSelect(target);
    };

    window.addEventListener('mouseout', mouseOutCallback, useCapture);
    window.addEventListener('mouseover', mouseOverCallback, useCapture);
    window.addEventListener('click', mouseClickCallback, useCapture);

    return () => {
      window.removeEventListener('mouseout', mouseOutCallback, useCapture);
      window.removeEventListener('mouseover', mouseOverCallback, useCapture);
      window.removeEventListener('click', mouseClickCallback, useCapture);
    };
  }, [
    elementRef.current,
    activeRef,
    ignoreElement,
    selectedElement,
    onlySelect,
  ]);

  useEffect(() => {
    if (activeRef) appendGlobalClassName(activeRef, tagName);
  }, [activeRef]);

  const TagName = tagName || 'element-highlight';

  return children ? <TagName ref={elementRef}>{children}</TagName> : null;
};

Highlight.disposeOfElement = element => {
  cleanupSelected();
  if (element) {
    element.classList.remove(SELECTED);
    element.classList.remove(ON_HOVER);
  }
};

export default Highlight;
